/*************************************************************************************************
*
*
*			ObjectMacZapp		-- a standard Mac OOP application template
*
*
*
*			ZAMDialog.cpp		-- a dialog box based on the appearance manager
*
*			
*
*
*
*			 1996, Graham Cox
*			Additional modifications/enhancements by Joe Strout.
*
*
*
*************************************************************************************************/


#include	"ZAMDialog.h"
#include	"MacZoop.h"

#include	<Appearance.h>

/*--------------------------------***  CONSTRUCTOR  ***---------------------------------*/


ZAMDialog::ZAMDialog( ZCommander* aBoss, const short dlogID )
	: ZDialog( aBoss, dlogID )
{
}

	
/*---------------------------------***  ACTIVATE  ***-----------------------------------*/
/*	
if an appearance manager savvy dialog, relies on ZWindow to perform activation
----------------------------------------------------------------------------------------*/

void	ZAMDialog::Activate()
{
	if ( gMacInfo.hasAppearanceMgr && IsAMDialog( macWindow ))
		ZWindow::Activate();
	else
		ZDialog::Activate();
}


/*--------------------------------***  DEACTIVATE  ***----------------------------------*/
/*	
if an appearance manager savvy dialog, relies on ZWindow to perform deactivation
----------------------------------------------------------------------------------------*/

void	ZAMDialog::Deactivate()
{
	if ( gMacInfo.hasAppearanceMgr && IsAMDialog( macWindow ))
		ZWindow::Deactivate();
	else
		ZDialog::Deactivate();
}


/*-----------------------------------***  TYPE  ***-------------------------------------*/
/*	
if an appearance manager savvy dialog, switches keyboard focus and handles typing using
new AM methods for these things.
----------------------------------------------------------------------------------------*/

void	ZAMDialog::Type( const char theKey, const short modifiers )
{
	if ( gMacInfo.hasAppearanceMgr && IsAMDialog( macWindow ))
	{
		DialogPeek	dp = (DialogPeek) macWindow;
		short		iType;
		Handle		iHand;
		Rect		iBox;
		
		Focus();
		
		if ( theKey == TAB_KEY )
			FailOSErr( AdvanceKeyboardFocus( macWindow ));
		else
		{
			ControlHandle	fc;
			short			partCode;
			EventRecord		theEvt;
			
			FailOSErr( GetKeyboardFocus( macWindow, &fc ));
			
			gApplication->GetCurrentEvent( &theEvt );
			partCode = HandleControlKey( fc, theEvt.message & keyCodeMask >> 8, theKey, theEvt.modifiers );
		
		}
		
		GetDialogItem( macWindow, dp->editField + 1, &iType, &iHand, &iBox );
		
		if (( iType & 0x80 ) == 0 )
			ClickItem( dp->editField + 1 );
	}
	else
		ZDialog::Type( theKey, modifiers );
}


/*-----------------------------------***  IDLE  ***-------------------------------------*/
/*	
if an appearance manager savvy dialog, calls the controls to idle them
----------------------------------------------------------------------------------------*/

void	ZAMDialog::Idle()
{
	if ( gMacInfo.hasAppearanceMgr && IsAMDialog( macWindow ))
		IdleControls( macWindow );
	else
		ZDialog::Idle();
}


/*-----------------------------------***  DRAW  ***-------------------------------------*/
/*	
if an appearance manager savvy dialog, calls new function to update the controls
----------------------------------------------------------------------------------------*/

void	ZAMDialog::Draw()
{
	if ( gMacInfo.hasAppearanceMgr && IsAMDialog( macWindow ))
		UpdateControls( macWindow, macWindow->visRgn );
	else
		ZDialog::Draw();
}


/*-----------------------------------***  CLICK  ***------------------------------------*/
/*	
if an appearance manager savvy dialog, uses new calls to track clicks, etc
----------------------------------------------------------------------------------------*/

void	ZAMDialog::Click( const Point mouse, const short modifiers )
{
	DialogPeek	dp = ( DialogPeek ) macWindow;
	
	// which item was hit?
	
	short	item = FindDialogItem( macWindow, mouse ) + 1;
	
	// if the item is enabled, call TrackControl, TEClick, etc as needed.
	
	if ( item > 0 )
	{
		short	iType, partCode;
		Handle	iHand;
		Rect	iBox;
		
		GetDialogItem( macWindow, item, &iType, &iHand, &iBox );

		if ( gMacInfo.hasAppearanceMgr && IsAMDialog( macWindow ))
		{
			ControlHandle 	cf;
			unsigned long	cFeatures;
			
			FailOSErr( GetKeyboardFocus( macWindow, &cf ));
			
			if (cf != (ControlHandle) iHand)
				FailOSErr( SetKeyboardFocus( macWindow, (ControlHandle) iHand, kControlFocusNextPart ));
			
			partCode = HandleControlClick((ControlHandle) iHand, mouse, modifiers, ( ControlActionUPP ) -1L );
			
			if (( partCode != 0 ) && (( iType & 0x80 ) == 0 ))
				ClickItem( item );
		}
		else
			ZDialog::Click( mouse, modifiers );
	}
}


/*--------------------------------***  ISAMDIALOG  ***----------------------------------*/
/*	
returns TRUE if a dialog is actually an Appearance Manager savvy dialog- i.e. one built
from 'dlog' or 'dlgx' resources rather than 'DLOG'. This works by seeing if the dialog
has a root control.
----------------------------------------------------------------------------------------*/

Boolean	IsAMDialog( DialogPtr aDialog )
{
	ControlHandle	rc;
	OSErr			theErr;
	
	theErr = GetRootControl( aDialog, &rc );
	
	return ( theErr == noErr );
}
