/*************************************************************************************************
*
*
*			MacZoop - "the framework for the rest of us"		 
*
*
*
*			ZGWorld.h			-- an object for managing an offscreen GWorld
*
*
*
*
*
*			 1996, Graham Cox
*
*
*
*
*************************************************************************************************/


#pragma once

#ifndef	__ZGWORLD__
#define __ZGWORLD__

#include	"ZObject.h"

#include	<QDOffscreen.h>
#include	<Quickdraw.h>
#include	<Icons.h>
#include	<FixMath.h>

// set up streaming stuff:

DEFINECLASSID( ZGWorld, 'zgwl' );

// class def:

class	ZGWorld	: public ZObject
{
protected:

	GWorldPtr		gw;
	GWorldFlags		pState;
	short			lockLevel;
	Boolean			inTempMem;

public:
	
	// constructors
	
	ZGWorld( const Rect& aSize,
			 const short aDepth,
			 const CTabHandle aCTable = NULL,
			 const Boolean makeTemp = FALSE );
			 
	ZGWorld( const PicHandle aPicture, const Boolean makeTemp = FALSE );
	ZGWorld( const short pictID, const Boolean makeTemp = FALSE );
	ZGWorld( ZGWorld* aGW, const Boolean makeTemp = FALSE );
	ZGWorld();
	
	// destructor
	
	virtual ~ZGWorld();
	
	// state manipulation and info
	
	virtual void			Lock();
	virtual void			Unlock();
	virtual void			SetPurgeable( const Boolean aPurgeState );
	virtual short			GetState();
	virtual Boolean			InMemory();
	
	// pixmap stuff
	
	virtual PixMapHandle	GetPixMap( const Boolean lock = FALSE );
	virtual void			ReallocPixMap();
	
	// setting the size, depth and colours
	
	virtual void			SetColours( const CTabHandle aCTable,
										const Boolean reMapColours = TRUE );
	virtual void			SetDepth( const short aDepth,
									  const CTabHandle aCTable = NULL,
									  const Boolean dither = FALSE );
	virtual void			SetSize( const Rect& aSize,
									 const Boolean scaleImage = TRUE );
	
	// getting info about the size, depth, colours, mem
	
	virtual short			GetDepth();
	virtual CTabHandle		GetColours();
	virtual void			GetSize( Rect* aFrame );
	virtual long			GetImageMemSize();
	virtual short			GetColourInfo();
	
	// copying utilities
	
	virtual void			CopyIn(	BitMap* srcBits, 	Rect* src, Rect* dest, short mode = srcCopy, RgnHandle mask = NULL );
	virtual void			CopyOut( BitMap* destBits, 	Rect* src, Rect* dest, short mode = srcCopy, RgnHandle mask = NULL ); 
	virtual void			CopyIn(	ZGWorld* srcGW, 	Rect* src, Rect* dest, short mode = srcCopy, RgnHandle mask = NULL );
	virtual void			CopyOut( ZGWorld* destGW, 	Rect* src, Rect* dest, short mode = srcCopy, RgnHandle mask = NULL ); 

	virtual void			Copy( ZGWorld* aGW, short mode = srcCopy );
	virtual void			Clear();
	
	// picture utilities
	
	virtual PicHandle		MakePicture( Rect* toFitRect = NULL, RgnHandle withMaskRgn = NULL );
	virtual void			CopyPicture( PicHandle aPicture, Rect* destRect = NULL );		
	
	// graphics port set-up
	
	virtual void			SetPortToGW( CGrafPtr* savedPort, GDHandle* savedDevice );
	virtual void			SetPortColours( RGBColor* aFore, RGBColor* aBack );
	virtual void			SetPortColours();
	virtual void			GetPortColours( RGBColor* aFore, RGBColor* aBack );
	
	// icon plotting
	
	virtual void			CopyIcon( CIconHandle anIcon, Rect* dest, short align = 0, short transform = 0 );
	virtual void			CopyIcon( short iconID, Rect* dest, short align = 0, short transform = 0 );
	
	// search procedures
	
	virtual void			InstallSearchProc( ColorSearchUPP aSearchProc );
	virtual void			RemoveSearchProc( ColorSearchUPP aSearchProc );
	
	// inversion & other special effects:
	
	virtual void			Invert( RgnHandle maskRgn = NULL );
	virtual void			FlipHorizontal( RgnHandle maskRgn = NULL );
	virtual void			FlipVertical( RgnHandle maskRgn = NULL );
	
	inline GWorldPtr		GetMacGWorld() { return gw; };
	virtual void			SetMacGWorld( GWorldPtr aGW );
	
	// streaming
	
	virtual void			WriteToStream( ZStream* aStream );
	virtual void			ReadFromStream( ZStream* aStream );

protected:
	
	virtual void			MakeMacGWorld(  const Rect& aSize,
											const short aDepth,
											const CTabHandle aCTable,
											const Boolean makeTemp );
	virtual void			MakeGWorldFromPicture( const PicHandle aPic, const Boolean makeTemp );
	virtual void			Flip( const short flipDir, RgnHandle mask );
};


// flip contants:

enum
{
	kFlipHorizontal = 1,
	kFlipVertical
};


// utility functions that can be used outside of this object if desired

void	GetPicParams( PicHandle aPic, Rect* aSize, short* aDepth, CTabHandle* aCTable );
void	Scale2Rects( Rect *theRect,Rect *refRect );
void	FlipRegion( RgnHandle aRgn, const short flipDir );


// some errors that could get thrown. Always be prepared to handle exceptions from this class.

#define			kGWNoMemForPixels		343
#define			kGWPixelsPurgedErr		344
#define			kGWCTableUpdtFailed		345
#define			kGWChangeDepthFailed	346
#define			kGWChangeSizeFailed		347
#define			kGWBadPictureErr		348
#define			kGWBadPictureSizErr		349

// colour depth info values

enum
{
	ctBlackAndWhite 		= 1,
	ctFourGreys 			= 2,
	ctSixteenGreys 			= 36,
	ctSixteenColours 		= 4,
	ct256Greys 				= 40,
	ct256Colours 			= 8,
	ctThousandsOfColours 	= 16,
	ctMillionsOfColours 	= 32
};

#endif