/************************************************************************************************
*
*
*		ColourPopUp.c		- high level call to work with PalMDEF function
*
*
*		15/7/94		1994 by Graham Cox
*
*
*************************************************************************************************/

/*
The PalMDEF function is a very useful piece of code, but is not very easy to work with because
the menu record has to be fiddled with in a rather disturbing way- get it wrong and you'll fall
over horribly. This code addresses that problem by wrapping up the ugly details of calling the 
MDEF in a nice high level function.

This file needs the PalMDEF file and the xDEFJump.c file
*/

#include	"ColourPopUp.h"
#include	"Palettes.h"
#include	"xDEFjump.h"


RGBColor	gCurrentPopUpColour = {0xDDDD,0xDDDD,0xDDDD};



long	PopUpColourTable( Point location, CTabHandle aCTable, RGBColor *selectedColour, short initialItem )
{
	// similar to palette function but accepts a CTable as a parameter & does not dispose it.

	CPMenuHandle	theMenu;
	short			menuID,itemID;
	long			mSelect = 0;
	Handle			mdefHandle;

	theMenu = (CPMenuHandle) NewHandleClear(sizeof(CPMenuRecord));
	
	if (theMenu)
	{
		mdefHandle = GetUniversalFunctionHandle((ProcPtr) PalMDEFFunction, uppMenuDefProcInfo );
		
		(*theMenu)->menuRecord.menuProc = mdefHandle;
		(*theMenu)->menuRecord.menuID = kColourPopUpMenuID;
		(*theMenu)->initIndex = initialItem;
				
		(*theMenu)->menuColours = aCTable;
		CalcMenuSize((MenuHandle) theMenu);
		InsertMenu((MenuHandle) theMenu,hierMenu);
		
		/* menu installed and ready to go, now we pop it up and track it */
		
		mSelect = PopUpMenuSelect((MenuHandle) theMenu,location.v,location.h,1);
		
		/* delete and dispose of the menu now we've finished with it */
		
		DeleteMenu(kColourPopUpMenuID);
		DisposeHandle(mdefHandle);
		(*theMenu)->menuRecord.menuProc = NULL;
		DisposeHandle((Handle) theMenu);
		
		
		/* now determine if we chose anything, and extract the colour if we did */
		
		menuID = HiWord(mSelect);
		itemID = LoWord(mSelect);
		
		if (menuID == kColourPopUpMenuID && aCTable)
			*selectedColour = (*aCTable)->ctTable[itemID -1].rgb;
	}
	return mSelect;
}


long	PopUpColourPalette(Point location,short cTableID,RGBColor *selectedColour,short initialItem)
{
	/*
	When you get a mouse down in the place you want to pop the menu up, call this function. The
	'clut' resource specified by the <cTableID> is loaded and displayed in the menu. The <location>
	parameter specifies the desired top,left point of the menu in global coordinates. The menu
	itself will be constrained by the monitor dimensions. The function returns the menuID in the
	high order word, and the index of the selected colour in the low order word. In addition,
	the chosen colour is returned in <selectedColour>. This saves you from reloading the clut to
	obtain the chosen colour. If no colour was picked, this result is undefined, and the function
	returns 0 as its result. Normally, the menu ID value should be ignored as it is created and
	destroyed on the fly by this function, and does not refer to a currently installed menu. Note
	that you are responsible for performing the highlighting (if any) of the clicked area.
	*/

	CTabHandle		theColours;
	long			mSelect = 0;
	
	theColours = GetCTable(cTableID);
	
	if (theColours)
	{
		mSelect = PopUpColourTable(location,theColours,selectedColour,initialItem);
		DisposeCTable(theColours);
	}
	return mSelect;
}


/*
The following function implements a standard appearance for a colour pop-up selector. This can
be called where needed to draw this box. The function takes two parameters, a rect and an RGBColor.
The function draws a pop-up arrow by loading the appropriate system resource.
*/


void	DrawStandardColourSelector(Rect *bounds,RGBColor *theColour)
{
	/* assumes current port */
	Rect			temp;
	RGBColor		saveBack;
	unsigned short	avgColour;
	
	temp = *bounds;
	temp.right--;
	temp.bottom--;
	GetBackColor(&saveBack);
	BackColor(whiteColor);
	EraseRect(&temp);
	RGBBackColor(&saveBack);
	FrameRect(&temp);
	
	MoveTo(temp.left + 2,temp.bottom);
	LineTo(temp.right,temp.bottom);
	LineTo(temp.right,temp.top + 2);
	
	InsetRect(&temp,3,3);
	RGBForeColor(theColour);
	PaintRect(&temp);
	
	avgColour = (((long) theColour->red + (long) theColour->green + (long) theColour->blue) / 3L );
	
	if ( avgColour > 0x4444 )
		ForeColor( blackColor );
	else
		ForeColor( whiteColor );
	
	temp.left = temp.right - 17;
	temp.top = temp.bottom - 15;
	
	DrawSICN(kStdPopUpArrowSICN,1,topLeft(temp));
	ForeColor( blackColor );
}

/* the following function is a dialog user item for drawing the standard colour popup. The
	RGBColour of the selector is passed as a global <gCurrentPopUpColour> */


pascal void	StandardColourUserItem(DialogPtr theDialog,short theItem)
{
	short		itemType;
	Handle		itemHand;
	Rect		itemBox;
	
	GetDialogItem(theDialog,theItem,&itemType,&itemHand,&itemBox);
	DrawStandardColourSelector(&itemBox,&gCurrentPopUpColour);
}


void	InvertStandardColourSelector(Rect *bounds)
{
	PenState	pnState;
	Rect		temp;
	
	GetPenState(&pnState);
	temp = *bounds;
	temp.right--;
	temp.bottom--;
	InsetRect(&temp,1,1);
	PenSize(2,2);
	PenMode(patXor);
	FrameRect(&temp);
	SetPenState(&pnState);
}


void	InstallColourUserItem(DialogPtr theDialog,short theItem)
{
	/* simple function to set up the colour popup user item */
	
	short				itemType;
	Handle				itemHand;
	Rect				itemBox;
	static UserItemUPP	myUserItemUPP;	

	if (theDialog)
	{
		GetDialogItem(theDialog,theItem,&itemType,&itemHand,&itemBox);
		
		if ((itemType & 0x7F) == userItem)
		{
			myUserItemUPP = NewUserItemProc((ProcPtr) StandardColourUserItem);
			itemHand = (Handle) myUserItemUPP;
			SetDialogItem(theDialog,theItem,itemType,itemHand,&itemBox);
		}
	}
}


/******************************************************************************
 DrawSICN

		Draw a SICN, a resource type which defines a 16 by 16 bit image,
		at the given location. SICN is specified by a resource ID number
		and a one-based index into the SICN list. The location specifies
		the top left corner of the SICN in the local coordinates of the
		current port.
 ******************************************************************************/

void	DrawSICN(short SICNid,short index,Point location)
{
	Handle		theSICN;
	BitMap		theImage;
	Rect		theBounds;

	theSICN = GetResource('SICN', SICNid);	
	if(theSICN)
	{

		HLock(theSICN);
											// A SICN resource is really a list
											//   of small icons, each of which
											//   has 32 bytes of data
											//   16 x 16 = 256 bits = 32 bytes
	
											// Create a bitmap so we can copy
											//   the SICN image on the screen
	
											// Index into handle to get a ptr
											//   to the SICN bit image
		theImage.baseAddr = ((char*)*theSICN) + (index - 1) * 32;
		theImage.rowBytes = 2;				// 16 bits is 2 bytes
	
											// Place bitmap at desired locaton
		SetRect(&theBounds, location.h, location.v,
							location.h + 16, location.v + 16);
		theImage.bounds = theBounds;
	
											// Copy image on to the screen
		CopyBits(&theImage, &qd.thePort->portBits,
				 &theBounds, &theBounds, srcOr, NULL);
	
		HUnlock(theSICN);
	}
}


void	UpdateDialogItem(DialogPtr theDialog,short theItem)
{
	/* updates the given dialog item */
	
	RgnHandle	temp,saveClip;
	short		itemType;
	Handle		itemHand;
	Rect		itemBox;
	GrafPtr		savePort;
	
	if (theDialog)
	{
		GetDialogItem(theDialog,theItem,&itemType,&itemHand,&itemBox);
		GetPort(&savePort);
		SetPort(theDialog);
		GetClip(saveClip = NewRgn());
		temp = NewRgn();
		
		RectRgn(temp,&itemBox);
		ClipRect(&itemBox);
		UpdateDialog(theDialog,temp);
		DisposeRgn(temp);
		
		SetClip(saveClip);
		DisposeRgn(saveClip);
		SetPort(savePort);
	}
}