/****************************************************************************************
*
*
*		ColourPalettePopUpMenu.MDEF
*
*		A menu definition procedure to create colour pop-up menus.
*		MOD 22/7/92 to improve look of menu and allow much easier selection of
*				binary colour chips.
*		This version 27/12/92 for MacMan II- implements more general palette style
*
****************************************************************************************/

#include 	"Menus.h"
#include	"Gestalt.h"
#include	"ColourPopUp.h"

#ifndef NIL
#define		NIL 	nil
#endif


#define		kDefaultSwatchWidth		17
#define		kDefaultSwatchHeight	17


static void	CalcCPMenuSize( MenuHandle theMenu, short chipsH, short chipsV );
static void CalcPopupMenuSize( Point hitPt, short item, Rect *mRect, short chipsH, short chipsV );
static void DrawCPMenu( MenuHandle theMenu, Rect *mBounds, short chipsH, short chipsV );
static void ChooseCPMenuItem( MenuHandle theMenu, short *theItem, Rect *mBounds, Point mHit, short chipsH, short chipsV );


pascal void	PalMDEFFunction(short message,MenuHandle theMenu,
							Rect *menuRect,Point hitPt,short *whichItem)
{
	/* main entry function. must be first function in the file */
	
	short			chipsH,chipsV;
	short			numSwatches,pDepth;
	long			gResponse;
	OSErr			theErr;
	GDHandle		gmScreen;
	CTabHandle		colours;
	static Boolean	hasBeenInited = FALSE;
	
	if (theMenu != NIL)
	{
		colours = (*(CPMenuHandle)theMenu)->menuColours;

		if (colours)
			numSwatches = (*colours)->ctSize + 1;
		else
		{
			theErr = Gestalt(gestaltQuickdrawVersion,&gResponse);
			if (theErr == noErr && (gResponse >= 0x0100))
			{
				gmScreen = GetMainDevice();
				if (gmScreen != NIL)
				{
					pDepth = (*(*gmScreen)->gdPMap)->pixelSize;
					switch(pDepth)
					{
						case 4:
							numSwatches = 16;
							break;
						case 8:
						case 16:
						case 32:
							numSwatches = 81;
							break;
						default:
							numSwatches = 2;
							break;
					}
				}
			}
			else
				numSwatches = 2;
		}
		switch (numSwatches)
		{
			case 256:
				chipsH = 16;
				break;
			default:
				chipsH = 9;
				break;
			case 16:
				chipsH = 4;
				break;
			case 2:
				chipsH = 2;
				break;
		}
		
		chipsV = numSwatches / chipsH;
		
		switch(message)
		{
			case mDrawMsg:
				DrawCPMenu(theMenu,menuRect,chipsH,chipsV,(*(CPMenuHandle)theMenu)->initIndex);
				hasBeenInited = FALSE;
				break;
			
			case mChooseMsg:
				if (hasBeenInited)
					ChooseCPMenuItem(theMenu,whichItem,menuRect,hitPt,chipsH,chipsV);
				else
				{
					if (PtInRect(hitPt,menuRect))
					{
						*whichItem = (*(CPMenuHandle)theMenu)->initIndex;
						hasBeenInited = TRUE;
					}
				}
				break;
			
			case mSizeMsg:
				CalcCPMenuSize(theMenu,chipsH,chipsV);
				break;
			
			case mPopUpMsg:
				CalcPopupMenuSize(hitPt,*whichItem,menuRect,chipsH,chipsV);
				break;
			
			default:
				break;
		}
	}
}


void CalcCPMenuSize(MenuHandle theMenu,short chipsH,short chipsV)
{
	/* calculates the size of the menu. This is determined by the size of each colour
		square, which is 16 * 16, with 1 pixel surrounding each */
	
	short swatchWidth,swatchHeight;
	
	if (chipsH <= 9)
		swatchWidth = swatchHeight = kDefaultSwatchWidth;
	else
		swatchWidth = swatchHeight = 12;

	(*theMenu)->menuWidth = swatchWidth * chipsH;
	(*theMenu)->menuHeight = swatchHeight * chipsV;
}


void CalcPopupMenuSize(Point hitPt,short item,Rect *mRect,short chipsH,short chipsV)
{
	/* calculates rectangle for pop-up menu. NOTE THAT THIS SHOULD be pinned to the frame of the
		monitor that contains the largest area */
		
	Rect 		sRect;
	GDHandle	mainScreen;
	short 		swatchWidth,swatchHeight;
	
	if (chipsH <= 9)
		swatchWidth = swatchHeight = kDefaultSwatchWidth;
	else
		swatchWidth = swatchHeight = 12;

	mRect->top = hitPt.h;	/* point order is reversed */
	mRect->left = hitPt.v;
	mRect->right = mRect->left + (swatchWidth * chipsH);
	mRect->bottom = mRect->top + (swatchHeight * chipsV);
	
	mainScreen = GetMainDevice();
	if (mainScreen)
	{
		sRect = (*(*mainScreen)->gdPMap)->bounds;
		InsetRect(&sRect,4,4);
	}
	if (mRect->right > sRect.right)
		OffsetRect(mRect,-(mRect->right - sRect.right),0); 
	
	if (mRect->bottom > sRect.bottom)
		OffsetRect(mRect,0,-(mRect->bottom - sRect.bottom)); 
}


void DrawCPMenu(MenuHandle theMenu,Rect *mBounds,short chipsH,short chipsV,short initItem)
{
	/* draws the menu. The colour info is taken from the menuinfo data structure, which
		is extended and the handle to the colour table is tacked on the end. */

	Rect		swatchRect;
	register	short			v,h;
	register	CTabHandle	colours;
	RGBColor	sColour;
	PenState	savePen;
	CGrafPtr	wmPort;
	GrafPtr		savePort;
	Ptr			fudge;
	short 		swatchWidth,swatchHeight;
	Point		zero = {0,0};
	
	if (chipsH <= 9)
		swatchWidth = swatchHeight = kDefaultSwatchWidth;
	else
		swatchWidth = swatchHeight = 12;
	
	colours = (*(CPMenuHandle)theMenu)->menuColours;
	
	if (colours)
	{
		SetRect(&swatchRect,0,0,swatchWidth,swatchHeight);
		OffsetRect(&swatchRect,mBounds->left,mBounds->top);
		if (chipsH <= 9)
			InsetRect(&swatchRect,2,2);

		for (v = 0;v < chipsV;v++)
		{
			for (h = 0;h < chipsH;h ++)
			{
				RGBForeColor(&(*colours)->ctTable[v * chipsV +h].rgb);
				PaintRect(&swatchRect);
				if (chipsH > 9)
					ForeColor(whiteColor);
				else
					ForeColor(blackColor);
				FrameRect(&swatchRect);
				OffsetRect(&swatchRect,swatchWidth,0);
			}
			OffsetRect(&swatchRect,-chipsH * swatchWidth,swatchHeight);
		}
		ForeColor(blackColor);
		
		ChooseCPMenuItem(theMenu,&initItem,mBounds,zero,chipsH,chipsV);
	}
}


void ChooseCPMenuItem(MenuHandle theMenu,short *theItem,Rect *mBounds,Point mHit,short chipsH,short chipsV)
{
	/* highlights the selection */
	
	Rect 			mRect,swatchRect,oldSwatchRect;
	short			sv,sh,item,oh,ov;
	PenState		penSave;
	short			numSwatches;
	short 			swatchWidth,swatchHeight;
	
	if (chipsH <= 9)
		swatchWidth = swatchHeight = kDefaultSwatchWidth;
	else
		swatchWidth = swatchHeight = 12;

	numSwatches = chipsH * chipsV;
	
	GetPenState(&penSave);
	PenMode(patXor);
	PenSize(2,2);
		
	mRect = *mBounds;
	InsetRect(&mRect,1,1);
	SetRect(&swatchRect,0,0,swatchWidth,swatchHeight);
	OffsetRect(&swatchRect,mBounds->left,mBounds->top);
	oldSwatchRect = swatchRect;
	
	if (PtInRect(mHit,&mRect))
	{
		/* mouse is in the menu, see which item it's in */
		
		for (sv = 0;sv < chipsV;sv ++)
		{
			for (sh = 0;sh < chipsH;sh ++)
			{
				if (PtInRect(mHit,&swatchRect))
					goto found;
				else
					OffsetRect(&swatchRect,swatchWidth,0);
			}
			OffsetRect(&swatchRect,-chipsH * swatchWidth,swatchHeight);
		}
	found:
		
		item = (sv * chipsV + sh) +1;
		if (item <= numSwatches && item != *theItem)
		{
			/* a new one, so unhighlight old one and hilite new one */
			
			if (*theItem != 0)
			{
				/* there really is an old one, so calc the hilite */
				oh = (*theItem -1) / chipsH;
				ov = (*theItem -1) % chipsH;
				OffsetRect(&oldSwatchRect,ov * swatchWidth,oh * swatchHeight);
				FrameRect(&oldSwatchRect);									/*Xor it out */
			}
			FrameRect(&swatchRect);											/*Xor it in */
			*theItem = item;
		}
	}
	else
	{
		if (*theItem != 0)
		{
			oh = (*theItem -1) / chipsH;
			ov = (*theItem -1) % chipsH;
			OffsetRect(&oldSwatchRect,ov * swatchWidth,oh * swatchHeight);
			FrameRect(&oldSwatchRect);
			
			*theItem = 0;
		}
	}	
	SetPenState(&penSave);
}
		

	