/***********************************************************************************************
*
*		Name:  			PatternMenu.c
*		Description:	A MDEF for displaying patterns, B & W or colour.
*
*
*		Date Created:		December 20th, 1995
*		Author:				Graham Cox
*		
*		Revision History:	
*
*
*
*		Notes:
*	
*
***********************************************************************************************/

#include	"PatternMenu.h"
#include	"ZDefines.h"
#include	"xDEFJump.h"

#define		kSwatchSize		32
#define		kSwatchOffset	2
#define		kMaxSwatchesH	6


static void DrawPatMenu(CPatMenuHandle theMenu,Rect* menuRect,short chipsH,short chipsV,short initIndex);
static void ChoosePatMenuItem(MenuHandle theMenu,short* whichItem,Rect* menuRect,Point hitPt,short chipsH,short chipsV);

static void CalcPatMenuSize(MenuHandle theMenu,short chipsH,short chipsV);
static void CalcPopPatMenuSize(Point hitPt,Rect* menuRect,short chipsH,short chipsV);



pascal void	CPatMDEFFunction(short message,MenuHandle theMenu,
								Rect *menuRect,Point hitPt,short *whichItem)
{
	short			chipsH,chipsV;
	short			numSwatches;
	static Boolean	isInited = FALSE;
	
	if (theMenu)
	{
		// set up the number of rows and columns in the menu
		
		if ((*(CPatMenuHandle) theMenu)->patListType == patListColour)
			numSwatches = (*(*(CPatMenuHandle) theMenu)->patListHdl)->numPatterns;
		else
			numSwatches = 36;
		
		chipsH = MIN(numSwatches,kMaxSwatchesH);
		chipsV = MAX(1,numSwatches / chipsH);
		
		switch(message)
		{
			case mDrawMsg:
				DrawPatMenu((CPatMenuHandle) theMenu,menuRect,chipsH,chipsV,(*(CPatMenuHandle)theMenu)->initIndex);
				isInited = FALSE;
				break;
			case mChooseMsg:
				if (isInited)
					ChoosePatMenuItem(theMenu,whichItem,menuRect,hitPt,chipsH,chipsV);
				else
				{
					if (PtInRect(hitPt,menuRect))
					{
						*whichItem = (*(CPatMenuHandle)theMenu)->initIndex;
						isInited = TRUE;
					}
				}
				break;
			case mSizeMsg:
				CalcPatMenuSize(theMenu,chipsH,chipsV);
				break;
			case mPopUpMsg:
				CalcPopPatMenuSize(hitPt,menuRect,chipsH,chipsV);
				break;
			default:
				break;
		}
	}
}



void CalcPatMenuSize(MenuHandle theMenu,short chipsH,short chipsV)
{
	// computes the menu's dimensions
	
	short swatchWidth,swatchHeight;
	
	swatchWidth = swatchHeight = kSwatchSize + kSwatchOffset;

	(*theMenu)->menuWidth = (swatchWidth * chipsH) + kSwatchOffset;
	(*theMenu)->menuHeight = (swatchHeight * chipsV) + kSwatchOffset;
}


void CalcPopPatMenuSize(Point hitPt,Rect* mRect,short chipsH,short chipsV)
{
	// dimensions for pop-up menu
	Rect 		sRect;
	GDHandle	mainScreen;
	short 		swatchWidth,swatchHeight;
	
	swatchWidth = swatchHeight = kSwatchSize + kSwatchOffset;

	mRect->top = hitPt.h;	/* point order is reversed */
	mRect->left = hitPt.v;
	mRect->right = mRect->left + (swatchWidth * chipsH) + kSwatchOffset;
	mRect->bottom = mRect->top + (swatchHeight * chipsV) + kSwatchOffset;
	
	mainScreen = GetMainDevice();
	if (mainScreen)
	{
		sRect = (*(*mainScreen)->gdPMap)->bounds;
		InsetRect(&sRect,4,4);
	}
	if (mRect->right > sRect.right)
		OffsetRect(mRect,-(mRect->right - sRect.right),0); 
	
	if (mRect->bottom > sRect.bottom)
		OffsetRect(mRect,0,-(mRect->bottom - sRect.bottom)); 

}


void DrawPatMenu(CPatMenuHandle theMenu,Rect* menuRect,short chipsH,short chipsV,short initIndex)
{
	// draws the menu
	
	short			h,v;
	short 			swatchWidth,swatchHeight;
	Rect			sr;
	CPatListHdl		thePatH;
	PixPatHandle	cPat;
	Pattern			bwPat;
	Point			zero = {0,0};
	
	swatchWidth = swatchHeight = kSwatchSize;
	thePatH = (*theMenu)->patListHdl;
	
	SetRect(&sr,kSwatchOffset,kSwatchOffset,swatchWidth + kSwatchOffset,swatchHeight + kSwatchOffset);
	OffsetRect(&sr,menuRect->left,menuRect->top);
	PenNormal();
	ForeColor(blackColor);
	BackColor(whiteColor);
	EraseRect(menuRect);
	
	for (v = 0;v < chipsV;v++)
	{
		for (h = 0;h < chipsH;h++)
		{
			switch((*theMenu)->patListType)
			{
				case patListColour:	
					cPat = GetPixPat((*thePatH)->patID[v * chipsH + h]);
					if (cPat)
					{
						FillCRect(&sr,cPat);
						DisposePixPat(cPat);
					}
					break;
				case patListSysList:
					GetIndPattern(&bwPat,0,v * chipsH + h + 1);
					FillRect(&sr,&bwPat);
					break;
				case patListBW:
					GetIndPattern(&bwPat,(*theMenu)->patListID,v * chipsH + h + 1);
					FillRect(&sr,&bwPat);
					break;
				case patListBWColourised:
					RGBForeColor(&(*theMenu)->patFore);
					RGBBackColor(&(*theMenu)->patBack);
					GetIndPattern(&bwPat,(*theMenu)->patListID,v * chipsH + h + 1);
					FillRect(&sr,&bwPat);
					ForeColor(blackColor);
					BackColor(whiteColor);
					break;
				default:
					break;
			}
			FrameRect(&sr);
			OffsetRect(&sr,swatchWidth + kSwatchOffset,0);
		}
		OffsetRect(&sr,-chipsH * (swatchWidth + kSwatchOffset),swatchHeight + kSwatchOffset);
	}
	
	ChoosePatMenuItem((MenuHandle) theMenu,&initIndex,menuRect,zero,chipsH,chipsV);
}


void ChoosePatMenuItem(MenuHandle theMenu,short* whichItem,Rect* menuRect,Point hitPt,short chipsH,short chipsV)
{
	// tracks mouse in menu
	Rect 			mRect,swatchRect,oldSwatchRect;
	short			sv,sh,item,oh,ov;
	PenState		penSave;
	short			numSwatches;
	short 			swatchWidth,swatchHeight;
	
	swatchWidth = swatchHeight = kSwatchSize + kSwatchOffset;

	numSwatches = chipsH * chipsV;
	
	GetPenState(&penSave);
	PenMode(patXor);
	PenSize(kSwatchOffset + 1,kSwatchOffset + 1);
		
	mRect = *menuRect;
	InsetRect(&mRect,1,1);
	SetRect(&swatchRect,0,0,swatchWidth + kSwatchOffset,swatchHeight + kSwatchOffset);
	OffsetRect(&swatchRect,menuRect->left,menuRect->top);
	oldSwatchRect = swatchRect;
	
	if (PtInRect(hitPt,&mRect))
	{
		/* mouse is in the menu, see which item it's in */
		
		for (sv = 0;sv < chipsV;sv ++)
		{
			for (sh = 0;sh < chipsH;sh ++)
			{
				if (PtInRect(hitPt,&swatchRect))
					goto found;
				else
					OffsetRect(&swatchRect,swatchWidth,0);
			}
			OffsetRect(&swatchRect,-chipsH * swatchWidth,swatchHeight);
		}
	found:
		
		item = (sv * chipsH + sh) +1;
		if (item <= numSwatches && item != *whichItem)
		{
			/* a new one, so unhighlight old one and hilite new one */
			
			if (*whichItem != 0)
			{
				/* there really is an old one, so calc the hilite */
				oh = (*whichItem -1) % chipsH;
				ov = (*whichItem -1) / chipsH;
				OffsetRect(&oldSwatchRect,oh * swatchWidth,ov * swatchHeight);
				FrameRect(&oldSwatchRect);									/*Xor it out */
			}
			FrameRect(&swatchRect);											/*Xor it in */
			*whichItem = item;
		}
	}
	else
	{
		if (*whichItem != 0)
		{
			oh = (*whichItem -1) % chipsH;
			ov = (*whichItem -1) / chipsH;
			OffsetRect(&oldSwatchRect,oh * swatchWidth,ov * swatchHeight);
			FrameRect(&oldSwatchRect);
			
			*whichItem = 0;
		}
	}	
	SetPenState(&penSave);
}







long	PopUpCPatternMenu(Point globalLoc,short patListID,short initialItem)
{
	CPatMenuHandle	theMenu;
	CPatListHdl		thePats;
	Handle			mdefHandle;
	long			mSelect = 0;
	
	theMenu = (CPatMenuHandle) NewHandleClear(sizeof(CPatMenuRecord));
	
	if (theMenu)
	{
		mdefHandle = GetUniversalFunctionHandle((ProcPtr) CPatMDEFFunction,uppMenuDefProcInfo);
		(*theMenu)->menuRecord.menuProc = mdefHandle;
		(*theMenu)->menuRecord.menuID = kPatternMenuID;
		(*theMenu)->initIndex = initialItem;
			
		thePats = GetCPatternList(patListID);
		(*theMenu)->patListHdl = thePats;
		(*theMenu)->patListID = patListID;
		(*theMenu)->patListType = patListColour;
		
		CalcMenuSize((MenuHandle) theMenu);
		InsertMenu((MenuHandle) theMenu,hierMenu);
		
		mSelect = PopUpMenuSelect((MenuHandle) theMenu,globalLoc.v,globalLoc.h,1);
		
		DeleteMenu(kPatternMenuID);
		ReleaseResource((Handle) thePats);
		DisposeHandle((Handle) theMenu);
		DisposeHandle(mdefHandle);
	}
	return mSelect;
}


long	PopUpBWPatternMenu(Point globalLoc,short patListID,short initialItem)
{
	CPatMenuHandle	theMenu;
	CPatListHdl		thePats;
	Handle			mdefHandle;
	long			mSelect = 0;

	theMenu = (CPatMenuHandle) NewHandleClear(sizeof(CPatMenuRecord));
	
	if (theMenu)
	{
		mdefHandle = GetUniversalFunctionHandle((ProcPtr) CPatMDEFFunction,uppMenuDefProcInfo);
		(*theMenu)->menuRecord.menuProc = mdefHandle;
		(*theMenu)->menuRecord.menuID = kPatternMenuID;
		(*theMenu)->initIndex = initialItem;
			
		(*theMenu)->patListHdl = NULL;
		(*theMenu)->patListID = patListID;
		if (patListID == 0)
			(*theMenu)->patListType = patListSysList;
		else
			(*theMenu)->patListType = patListBW;
		
		CalcMenuSize((MenuHandle) theMenu);
		InsertMenu((MenuHandle) theMenu,hierMenu);
		
		mSelect = PopUpMenuSelect((MenuHandle) theMenu,globalLoc.v,globalLoc.h,1);
		
		DeleteMenu(kPatternMenuID);
		DisposeHandle((Handle) theMenu);
		DisposeHandle(mdefHandle);
	}
	return mSelect;
}


long	PopUpBWColourisedMenu(Point globalLoc,short patListID,short initialItem,RGBColor* fClr,RGBColor* bClr)
{
	CPatMenuHandle	theMenu;
	CPatListHdl		thePats;
	Handle			mdefHandle;
	long			mSelect = 0;

	theMenu = (CPatMenuHandle) NewHandleClear(sizeof(CPatMenuRecord));
	
	if (theMenu)
	{
		mdefHandle = GetUniversalFunctionHandle((ProcPtr) CPatMDEFFunction,uppMenuDefProcInfo);
		(*theMenu)->menuRecord.menuProc = mdefHandle;
		(*theMenu)->menuRecord.menuID = kPatternMenuID;
		(*theMenu)->initIndex = initialItem;
			
		(*theMenu)->patListHdl = NULL;
		(*theMenu)->patListID = patListID;
		(*theMenu)->patListType = patListBWColourised;
		
		(*theMenu)->patFore = *fClr;
		(*theMenu)->patBack = *bClr;
		
		CalcMenuSize((MenuHandle) theMenu);
		InsertMenu((MenuHandle) theMenu,hierMenu);
		
		mSelect = PopUpMenuSelect((MenuHandle) theMenu,globalLoc.v,globalLoc.h,1);
		
		DeleteMenu(kPatternMenuID);
		DisposeHandle((Handle) theMenu);
		DisposeHandle(mdefHandle);
	}
	return mSelect;
}


CPatListHdl		GetCPatternList(short patListID)
{
	return (CPatListHdl) GetResource(kCPatListResType,patListID);
}


PixPatHandle	GetIndCPattern(short patListID, short index)
{
	// obtains the actual pattern handle from a 'PPT#' resource with ID and index, or NULL
	// if anything wasn't found.
	
	PixPatHandle	aCpat = NULL;
	CPatListHdl		cList;
	short			patID;
	
	cList = GetCPatternList(patListID);
	
	if (cList)
	{
		// does the list contain the index we want?
		
		if (index > 0 && index <= (*cList)->numPatterns)
		{
			patID = (*cList)->patID[index - 1];
			aCpat = GetPixPat(patID);
		}
		ReleaseResource((Handle) cList);
	}
	return aCpat;
}



