/*************************************************************************************************
*
*
*			ObjectMacZapp			-- a standard Mac OOP application template
*
*
*
*			ZLMListWindow.cpp		-- a window that can display a List Manager list
*
*
*
*
*
*			 1996, Graham Cox
*
*
*
*
*************************************************************************************************/



#include	"ZLMListWindow.h"
#include	"ZErrors.h"
#include	"ZDefines.h"
#include	"xDEFJump.h"

#include	<LowMem.h>

// static function that calls us back from the custom LDEF stub we install. This allows the
// drawing of the list cells to be overridden like any other method. Cool, huh?

static pascal void	LMListDefVectorProc(	short message,
											Boolean selected,
											Rect* cellRect,
											Cell theCell,
											short dataOffset,
											short dataLen,
											ListHandle theList);
											
static pascal Boolean	LMListClickLoopProc();

static ListHandle	gClickList = NULL;


/*---------------------------------***  CONSTRUCTOR  ***--------------------------------*/


ZLMListWindow::ZLMListWindow( ZCommander* aBoss, short windID, short rows, short cols )
	: ZWindow( aBoss, windID )
{
	itsRows = rows;
	itsCols = cols;
	inSizeOp = FALSE;
	
	cellSize.h = cellSize.v = 0;
	
	hasVBar = TRUE;
	hasHBar = ( cols > 1 );
}

/*---------------------------------***  DESTRUCTOR  ***---------------------------------*/

ZLMListWindow::~ZLMListWindow()
{
	if ( itsList )
	{
		Handle	defProcH = (*itsList)->listDefProc;
		
		if ((*itsList)->lClickLoop )
		{
			DisposeRoutineDescriptor((*itsList)->lClickLoop );
			(*itsList)->lClickLoop = NULL;
		}
		
		LDispose( itsList );
		
		if ( defProcH )
			DisposeHandle( defProcH );	
	}
}


/*--------------------------------***  INITZWINDOW  ***---------------------------------*/
/*
Initialise the normal stuff, then the list manager, inclusing vector LDEF	
----------------------------------------------------------------------------------------*/

void	ZLMListWindow::InitZWindow()
{
	ZWindow::InitZWindow();
	
	ListManagerInit();
}


/*--------------------------------***  DRAWCONTENT  ***---------------------------------*/
/*
Update the list
----------------------------------------------------------------------------------------*/

void	ZLMListWindow::DrawContent()
{
	if ( !inSizeOp )
	{
		if ( itsList )
			LUpdate( qd.thePort->visRgn, itsList );
	}
}


/*-----------------------------------***  CLICK  ***------------------------------------*/
/*
Pass on clicks to the list manager, which handles everything itself
----------------------------------------------------------------------------------------*/

void	ZLMListWindow::Click( Point mouse, short modifiers )
{
	if ( itsList )
	{
		gClickList = itsList;
		
		if ( LClick( mouse, modifiers, itsList ))
		{
			// if it was a double-click, determine which cell it was and
			// pass this to the CellDoubleClick method for processing
			
			Cell aCell = LLastClick( itsList );
		
			CellDoubleClick( aCell );
		}	
	}
}


/*----------------------------------***  ACTIVATE  ***----------------------------------*/
/*
Activate the list, then the window
----------------------------------------------------------------------------------------*/

void	ZLMListWindow::Activate()
{
	if ( itsList )
		LActivate( TRUE, itsList );
		
	ZWindow::Activate();
}


/*--------------------------------***  DEACTIVATE  ***---------------------------------*/
/*
Deactivate the window, then the list	
----------------------------------------------------------------------------------------*/

void	ZLMListWindow::Deactivate()
{
	ZWindow::Deactivate();
	
	if ( itsList )
		LActivate( FALSE, itsList );
}


/*----------------------------------***  SETSIZE  ***-----------------------------------*/
/*
set the list to the same size as the window (moves scrollbars, etc)
----------------------------------------------------------------------------------------*/

void	ZLMListWindow::SetSize( short width, short height, Boolean reDraw )
{
	inSizeOp = TRUE;	// don't draw twice (Draw suppressed if TRUE)
	
	ZWindow::SetSize( width, height, reDraw );
	
	if ( hasVBar || hasHBar )
	{
		width -= kScrollBarWidth;
		height -= kScrollBarWidth;
	}
	
	if ( itsList )
		LSize( width, height, itsList );
	
	inSizeOp = FALSE;
	
	Focus();
	Draw();
	ValidRect( &macWindow->portRect );
}

/*------------------------------------***  ZOOM  ***------------------------------------*/
/*
Ditto for the zoom case
----------------------------------------------------------------------------------------*/

void	ZLMListWindow::Zoom( short partCode )
{
	short	w, h;
	
	ZWindow::Zoom( partCode );
	
	w = macWindow->portRect.right - macWindow->portRect.left;
	h = macWindow->portRect.bottom - macWindow->portRect.top;
	
	if ( hasVBar || hasHBar )
	{
		w -= kScrollBarWidth;
		h -= kScrollBarWidth;
	}
		
	if ( itsList )
		LSize( w, h, itsList );
	
	Focus();	
	Draw();
	ValidRect( &macWindow->portRect );
}


/*---------------------------------***  APPENDROW  ***----------------------------------*/
/*
add one row to the bottom of the list- returns the row number of the added row	
----------------------------------------------------------------------------------------*/

short	ZLMListWindow::AppendRow()
{
	short	rn = -1;
	
	if (itsList)
	{
		short	dn = (*itsList)->dataBounds.bottom + 1;
		
		// check that list is not too big
		
		if ((dn * (*itsList)->cellSize.v) > 32760 )
			FailOSErr( kListTooBigErr );
		
		rn = LAddRow( 1, dn, itsList );
		
		CalcBounds();
	}
	return rn;
}

/*---------------------------------***  APPENDCOL  ***----------------------------------*/
/*
add one column to the right of the list- returns column number of the added column	
----------------------------------------------------------------------------------------*/

short	ZLMListWindow::AppendCol()
{
	short	rn = -1;
	
	if (itsList)
	{
		short	dn = (*itsList)->dataBounds.right + 1;
		
		// check that list is not too big
		
		if ((dn * (*itsList)->cellSize.h) > 32760 )
			FailOSErr( kListTooBigErr );
		
		rn = LAddColumn( 1, dn, itsList );
		
		CalcBounds();
	}
	return rn;
}

/*-------------------------------***  GETROWCOLINFO  ***--------------------------------*/
/*
return sthe number of rows and columns in the list at present	
----------------------------------------------------------------------------------------*/

void	ZLMListWindow::GetRowColInfo( short* numRows, short* numCols )
{
	if ( itsList )
	{
		*numRows = (*itsList)->dataBounds.bottom;;
		*numCols = (*itsList)->dataBounds.right;
	}
}


/*----------------------------------***  SETDATA  ***-----------------------------------*/
/*
Sets the data of <aCell> to the string passed. If the data is not string data, use the
other form of SetData. This method automatically appends a row or column if the cell passed
is beyond the current range of the list. This allows you to build up a list easily by
making repeated calls to SetData, instead of a two-step process to make the cell then
set the data.	
----------------------------------------------------------------------------------------*/

void	ZLMListWindow::SetData(Cell aCell, Str255 sData)
{
	if ( itsList )
	{
		if (aCell.v > (*itsList)->dataBounds.bottom )
			aCell.v = AppendRow();
		
		if (aCell.h > (*itsList)->dataBounds.right )
			aCell.h = AppendCol();
			
		LSetCell( &sData[1], sData[0], aCell, itsList );
	}
}


/*----------------------------------***  SETDATA  ***-----------------------------------*/
/*
Sets the cell data using a pointer and length, which may be more appropriate for custom
lists. In the common case of listing a string however, you can use SetData with a String.
----------------------------------------------------------------------------------------*/

void	ZLMListWindow::SetData( Cell aCell, Ptr sData, short dataLen )
{
	if ( itsList )
	{
		if (aCell.v > (*itsList)->dataBounds.bottom )
			aCell.v = AppendRow();
		
		if (aCell.h > (*itsList)->dataBounds.right )
			aCell.h = AppendCol();
		
		LSetCell( sData, dataLen, aCell, itsList );
	}
}


/*-----------------------------------***  GETDATA  ***----------------------------------*/
/*
Gets the cells data into an arbitrary pointer. This does no checking, so be careful. The
value of dataLen must be set on entry. The actuall amount returned is passed back on
exit. sData must point to somewhere valid, with enough space to store the expected data.
For the more common case of getting a string, use the other form of GetData.
----------------------------------------------------------------------------------------*/

void	ZLMListWindow::GetData( Cell aCell, Ptr sData, short* dataLen )
{
	if ( itsList )
		LGetCell( sData, dataLen, aCell, itsList );
}

/*-----------------------------------***  GETDATA  ***----------------------------------*/
/*
Gets the data of <aCell> into the string passed. If the data is not string data, use the
other form of GetData	
----------------------------------------------------------------------------------------*/

void	ZLMListWindow::GetData( Cell aCell, Str255 sData )
{
	short	sLen = 255;
	
	if ( itsList )
	{
		LGetCell( &sData[1], &sLen, aCell, itsList );
		
		sData[0] = sLen & 0xFF;
	}
}

/*---------------------------------***  DRAW1CELL  ***----------------------------------*/
/*
called from the vector LDEF to draw a cell. You can override this to create custom lists.
This method does pretty much same as the standard system LDEF.	
----------------------------------------------------------------------------------------*/

void	ZLMListWindow::Draw1Cell( Cell aCell, Rect* area )
{
	Str255	aString;
	short	sLen = 255;
	
	EraseRect( area );
	
	MoveTo( area->left + (*itsList)->indent.h, area->top + (*itsList)->indent.v );
	LGetCell( &aString[1], &sLen, aCell, itsList );
	aString[0] = sLen & 0xFF;
	DrawString( aString );
	
	#ifdef _DRAW_LINES_BETWEEN_CELLS
	MoveTo( area->left, area->top + (*itsList)->cellSize.v - 1 );
	PenPat( &qd.gray );
	PenSize( 1, 1 );
	LineTo( area->right, area->top + (*itsList)->cellSize.v - 1 );
	
	PenNormal();
	#endif
}

/*--------------------------------***  HILITECELL  ***----------------------------------*/
/*
highlight the cell passed. Override this to do custom highlighting.	
----------------------------------------------------------------------------------------*/

void	ZLMListWindow::HiliteCell( Cell aCell, Rect* area )
{
	SetHiliteMode();
	InvertRect( area );	
}

/*-------------------------------***  SETCELLSIZE  ***----------------------------------*/
/*
sets the cell size for the list to that passed. Note that to be effective, this must be
called after (or inside) the constructor but before InitZWindow.
----------------------------------------------------------------------------------------*/

void	ZLMListWindow::SetCellSize( short aWidth, short aHeight )
{
	cellSize.h = aWidth;
	cellSize.v = aHeight;
}


/*-----------------------------***  SETLMSCROLLUSAGE  ***-------------------------------*/
/*
sets whether the list can horizontal or vertical scrollbars, or none. To be effective,
this should be called after the constructor but before InitZWindow.
----------------------------------------------------------------------------------------*/

void	ZLMListWindow::SetLMScrollUsage( Boolean aVBar, Boolean aHBar )
{
	hasHBar = aHBar;
	hasVBar = aVBar;
}

/*-----------------------------***  LISTMANAGERINIT  ***--------------------------------*/
/*
creates the list manager stuff and sets up the vector LDEF so that the object does the
list cell drawing and not the Mac toolbox. Called by InitZWindow.	
----------------------------------------------------------------------------------------*/

void	ZLMListWindow::ListManagerInit()
{
	Rect	r, db;
	
	// db is the databounds, which is initially set from itsRows and ItsCols
	
	SetRect( &db, 0, 0, itsCols, itsRows );
	r = macWindow->portRect;
	
	// subtract scrollbar area
	
	if ( hasVBar || hasHBar )
	{
		r.right -= kScrollBarWidth;
		r.bottom -= kScrollBarWidth;
	}
	
	FailNIL( itsList = LNew( &r, &db, cellSize, 0, macWindow, FALSE, hasHBar || hasVBar, hasHBar, hasVBar ));
	
	// the refCon of the list points to the this object
	
	(*itsList)->refCon = (long) this;
	
	// set up the handle to the vector proc
	
	#ifndef _NEVER_USE_CUSTOM_LDEF
	Handle	vpHand = GetUniversalFunctionHandle((ProcPtr) LMListDefVectorProc, uppListDefProcInfo);
	(*itsList)->listDefProc = vpHand;
	#endif
	
	// install a clickloop that calls us back
	
	ListClickLoopUPP	clikUPP = NewListClickLoopProc( LMListClickLoopProc );
	(*itsList)->lClickLoop = clikUPP;
	
	// if OK, turn on drawing
	
	LSetDrawingMode( TRUE, itsList );
}



void	ZLMListWindow::CalcBounds()	
{
	// sets the bounds of the window to the size of the list.
	
	Rect	rb;
	short	cw, ch;
	
	cw = cellSize.h;
	ch = cellSize.v;
	
	if (cw <= 0)
		cw = macWindow->portRect.right - macWindow->portRect.left - kScrollBarWidth;
		
	if (ch <= 0)
		ch = macWindow->portRect.bottom - macWindow->portRect.top - kScrollBarWidth;
	
	SetRect( &rb, 0, 0,
				cw * (*itsList)->dataBounds.right,
				ch * (*itsList)->dataBounds.bottom );
	
	rb.right = MAX( rb.right, cw );
	rb.bottom = MAX( rb.bottom, ch );
				
	bounds = rb;
	
	rb.bottom += kScrollBarWidth;
	rb.right += kScrollBarWidth;
	rb.top = ch;
	rb.left = 200;
	
	SetSizeRect( rb );
}



Boolean		ZLMListWindow::ClickLoop()
{
	// callback for clickloop- by default this does nothing. To abort LClick, return FALSE.


	return TRUE;
}

#pragma mark -

/*---------------------------***  LMLISTDEFVECTORPROC  ***------------------------------*/
/*
passes calls to the LDEF back to the object so that we can easily create simple overridable
custom list behaviours. This is pretty neat, since this is otherwise a bit of a chore.
----------------------------------------------------------------------------------------*/

static pascal void	LMListDefVectorProc(	short message,
											Boolean selected,
											Rect* cellRect,
											Cell theCell,
											short dataOffset,
											short dataLen,
											ListHandle theList)
{
	ZLMListWindow*	zw;
	
	zw = (ZLMListWindow*) (*theList)->refCon;
	
	if (zw)
	{
		try
		{
			switch (message)
			{
				case lDrawMsg:
					zw->Draw1Cell(theCell, cellRect);
					if (! selected)
						break;
				
				case lHiliteMsg:
					zw->HiliteCell(theCell, cellRect);
					break;
				
				default:
					break;
			}
		}
		catch(OSErr err)
		{
			// do not propagate!!!
		}
	}
}




static pascal Boolean	LMListClickLoopProc()
{
	ZLMListWindow*	zw;
	Boolean			result = TRUE;
	
	zw = (ZLMListWindow*) (*gClickList)->refCon;
	
	if (zw)
	{
		try
		{
			result = zw->ClickLoop();
		}
		catch( OSErr err )
		{
			result = FALSE;
		}
	}
	return result;
}