/*************************************************************************************************
*
*
*			ObjectMacZapp		-- a standard Mac OOP application template
*
*
*
*			ZSortTestWindow.cpp			-- a window that demonstrates the sort function
*
*
*
*
*
*			 1996, Graham Cox
*
*
*
*
*************************************************************************************************/


#include	"ZSortTestWindow.h"
#include	"MacZoop.h"
#include	"ZArray.h"
#include	"ZFile.h"
#include	"ZProgress.h"

#include	"ZHandleStream.h"
#include	"ZFileStream.h"

static short MyCompareFunc( void* a, void* b, long ref );
static short MyScrambleFunc( void* a, void* b, long ref );



CLASSCONSTRUCTOR( ZSortTestWindow );



ZSortTestWindow::ZSortTestWindow( ZCommander* aBoss, short windID )
	: ZScroller( aBoss, windID )
{
	// the array will hold Str63s. This is a seriously inefficient way to store lots
	// of strings, but this is only a test.
	
	FailNIL(itsArray = new ZArray( 63 ));
	GetDateTime((unsigned long*) &qd.randSeed );
	sortTime = -1;
	classID = CLASS_ZSortTestWindow;
}


ZSortTestWindow::ZSortTestWindow()
{
	itsArray = NULL;
	sortTime = -1;
	classID = CLASS_ZSortTestWindow;
}



ZSortTestWindow::~ZSortTestWindow()
{
	ForgetObject( itsArray );
}


void	ZSortTestWindow::InitZWindow()
{
	ZScroller::InitZWindow();
	
	printable = TRUE;
	
	Focus();
	TextFont( kFontIDMonaco );
	TextSize( 9 );
}


void	ZSortTestWindow::DrawContent()
{
	Rect		cr, lr;
	long		i, linePos;
	short		lineHeight;
	FontInfo	fi;
	Str255		s;
	
	if (isPrinting )
		GetBounds( &cr );
	else
	{
		GetContentRect( &cr );
		EraseRect( &cr );
	}
	// calculate the lineheight
	
	GetFontInfo( &fi );
	lineHeight = fi.ascent + fi.descent + fi.leading;
	
	linePos = fi.ascent;
	
	// draw some stats up front
	
	MoveTo( 8, linePos );		
	DrawString("\pNumber of items: ");
	NumToString( itsArray->CountItems(), s );
	DrawString( s );
	
	if (sortTime != -1)
	{
		DrawString("\p. Sort/Scramble took about ");
		NumToString( sortTime, s );
		DrawString( s );
		DrawString("\p tick");
		if (sortTime != 1)
			DrawChar('s');
	}
	
	PenPat( &qd.gray );
	linePos += lineHeight;
	MoveTo( 0, linePos - 3);
	LineTo( cr.right, linePos - 3);		
	linePos += lineHeight;
	PenNormal();
	
	// draw each string to the window
	
	for (i = 1; i<= itsArray->CountItems(); i++)
	{
		MoveTo( 8, linePos );		
		
		// calculate the line rect and only draw strings that intersect the
		// content rect. (This is just an optimisation for faster drawing).
		
		SetRect(&lr, 0, linePos - fi.ascent, cr.right, linePos + fi.descent + fi.leading);
		
		if (SectRect( &lr, &cr, &lr ))
		{
			itsArray->GetArrayItem((void*) s, i);
			DrawString( s );
		}
		linePos += lineHeight;
	}
}



void	ZSortTestWindow::UpdateMenus()
{
	gMenuBar->EnableCommand( kCmdSort );
	gMenuBar->EnableCommand( kCmdScramble );
	
	ZScroller::UpdateMenus();
}


void	ZSortTestWindow::HandleCommand( const long aCmd )
{
	switch ( aCmd )
	{
		case kCmdSort:
			SetBeachBallCursor();
			sortTime = TickCount();
			itsArray->Sort( MyCompareFunc, (long) this );
			sortTime = TickCount() - sortTime;
			PostRefresh();
			dirty = TRUE;
			break;
		
		case kCmdScramble:
			SetBeachBallCursor();
			sortTime = TickCount();
			itsArray->Sort( MyScrambleFunc, (long) this );
			sortTime = TickCount() - sortTime;
			PostRefresh();
			dirty = TRUE;
			break;
			
	}

	ZScroller::HandleCommand( aCmd );
}


void	ZSortTestWindow::OpenFile( const OSType aType, Boolean isStationery )
{
	// builds the initial array by reading a file of strings. This is a text file called
	// "test_strings.txt". The code here should not be regarded as a particularly good
	// way to read strings from a file- it is a quick and dirty implementation that is
	// far from optimum. The data in the file is actually scanned twice for example, which
	// could be easily avoided with a little work.
	
	ZFile		aFile( macFile );
	long		bytes, prevMark;
	short		len, c;
	char		fChar;
	Str255		fStr;
	
	aFile.Open();
	
	ZProgress	zPD( this, kStdProgressResID, aFile.GetLength(), kProportionalProgress, kCancelType );
	zPD.SetDelay( 60 );
	zPD.SetMessage("\pReading text file...");
	zPD.SetTitle( macFile.name );
	
	try
	{
		// discard any existing items in the array by removing it
		// and making a new one
		
		ForgetObject( itsArray );
		FailNIL( itsArray = new ZArray( 63 ));
		
		// scan for strings. We look for a return (0x0D) or the char count going
		// over what we can store in one array entry (63 chars).
		
		while( true )
		{
			prevMark = aFile.GetMark();
			fChar = 0;
			bytes = 1;
			c = 0;

			while((fChar != 0x0D) && (++c < 63))
				aFile.Read( &fChar, &bytes );
				
			// found CR or string of max length, so get the length of that string
			
			len = aFile.GetMark() - prevMark;
			
			// if len is only 1 or less, skip it- we don't want lots of empty strings!
			
			if (len > 1)
			{
				// extract the string from the file. Be careful that we don't go off the end
				// of the string- the local string buffer has 255 bytes, the array can store 62.
				
				aFile.SetMark( prevMark );
				bytes = MIN( 255, len);
				aFile.Read((Ptr) &fStr[1], &bytes );
				fStr[0] = MIN( 63, len);
				
				aFile.SetMark( prevMark + len );
				
				// set string into the array
				
				itsArray->AppendItem( fStr );
			}
			// update the progress indicator
			
			if (! zPD.InformProgress( aFile.GetMark()))
				FailOSErr( userCanceledErr );	
		}
	}
	catch( OSErr err )
	{
		// it was probably only the end-of-file, or the user cancelling, in which
		// case carry on as normal. Other errors are thrown.
		
		if (err != eofErr)
			throw err;
	}
	
	CalcBounds();
	
	// inherited method sets title of window, etc
	
	ZScroller::OpenFile( aType, isStationery );
}


void	ZSortTestWindow::CalcBounds()
{
	// calculate the scrollbounds. Again, this is crude, since a file with more than
	// about 2,500 strings will cause the coordinates to overflow. We take no pre-
	// cautions against this, so things may become a bit odd if this occurs.
	
	FontInfo	fi;
	Rect		sb;
	
	Focus();
	GetFontInfo( &fi );
	SetRect(&sb, 0, 0,
			(63 * fi.widMax) + 8,
			((fi.ascent + fi.descent + fi.leading) * (itsArray->CountItems() + 2)) + 1); 		
	SetBounds( sb );
	SetScrollAmount( fi.widMax, fi.ascent + fi.descent + fi.leading); 
}



void	ZSortTestWindow::WriteToStream( ZStream* aStream )
{
	ZScroller::WriteToStream( aStream );
	aStream->WriteObject( itsArray );
}



void	ZSortTestWindow::ReadFromStream( ZStream* aStream )
{
	ZScroller::ReadFromStream( aStream );
	ForgetObject( itsArray );
	
	itsArray = (ZArray*) aStream->ReadObject();
	
	CalcBounds();
}


#pragma mark -

static short MyCompareFunc( void* a, void* b, long ref )
{
	// a and b are pointers to Str255's. We can simply call the toolbox to compare them.
	
	return RelString((ConstStr255Param) a, (ConstStr255Param) b, TRUE, TRUE );
}



static short MyScrambleFunc( void* a, void* b, long ref )
{
	// scrambles the sort order by returning a random value

	return ( ABS(Random()) % 3 ) - 1;
}

